scriptname INN_RoomCostsConfig extends SKI_ConfigBase

import INN_Data

INN_RoomCostsScript property InnScript auto

FormList property InnCells auto
{ A list of the inn interior cells. }

FormList property InnLocations auto
{ A list of the inn locations.  Note: these are the parent locations, not the interior cell locations.  Used only for MCM display purposes. }

Message property MsgNoPapyrusUtil auto

int idDebugMode
int idDefaultRoomCost
int idUseCheckoutTime
int idUseColoredLabels
int idCheckoutTime
int idCheckoutTime2
int[] idInn

bool bNewGame = true

;-------------------------------------------------------------------------------

string function FormatTime(float afTime)
	string text = ""
	float fValue = afTime
	bool bPM = false
	if (fValue >= 12.0)
		bPM = true
		fValue -= 12.0
	endif
	int iHour = Math.Floor(fValue)
	fValue -= iHour
	if (iHour == 0)
		iHour = 12
	endif
	text += iHour + ":"
	int iMin = Math.Floor(100.0 * fValue + 0.5)
	if (iMin == 0)
		text += "00"
	elseif (iMin == 25)
		text += "15"
	elseif (iMin == 50)
		text += "30"
	else
		text += "45"
	endif
	if (bPM)
		text += " PM"
	else
		text += " AM"
	endif
	return text
endfunction

;-------------------------------------------------------------------------------

string function GetInnDisplayName(string sInn)
	string text
	if (UseColoredLabels() && GetVisited(sInn))
		text = "<font color='#33dd66'>" + sInn + "</font>"
	else
		text = sInn
	endif
	return text
endfunction

;-------------------------------------------------------------------------------

string function GetInnName(int aiOption)
	int index = aiOption - idInn[0]
	string sInn = ""
	if (CurrentPage == "$InnRoomCosts_Vanilla")
		sInn = VanillaInnsGet(index)
	elseif (CurrentPage == "$InnRoomCosts_Mods")
		sInn = ModInnsGet(index)
	endif
    return sInn
endfunction

;-------------------------------------------------------------------------------

int function GetVersion()
	return 4
endfunction

;-------------------------------------------------------------------------------

function LoadData()
	if (JsonUtil.JsonExists(GetModName()))
		;/
			Load the PapyrusUtil JSON file
		/;
		JsonUtil.Load(GetModName())
	elseif (IsJContainersInstalled())
		;/
			Check for a JContainers JSON file and if one exists then migrate
			the data from it.
		/;
		int iData = JValue.readFromFile("Data/SKSE/Plugins/JCData/InnRoomCosts.json")
		if (iData != 0)
			JDB.setObj("InnRoomCosts", iData)
			if (JDB.hasPath(".InnRoomCosts.CheckoutTime"))
				SetCheckoutTime(JDB.solveFlt(".InnRoomCosts.CheckoutTime"))
			endif
			if (JDB.hasPath(".InnRoomCosts.DebugMode"))
				SetDebugMode(JDB.solveInt(".InnRoomCosts.DebugMode"))
			endif
			if (JDB.hasPath(".InnRoomCosts.DefaultRoomCost"))
				SetDefaultRoomCost(JDB.solveInt(".InnRoomCosts.DefaultRoomCost"))
			endif
			if (JDB.hasPath(".InnRoomCosts.UseColoredLabels"))
				SetUseColoredLabels(JDB.solveInt(".InnRoomCosts.UseColoredLabels"))
			endif
			if (JDB.hasPath(".InnRoomCosts.UseCheckoutTime"))
				SetUseCheckoutTime(JDB.solveInt(".InnRoomCosts.UseCheckoutTime"))
			endif
			if (JDB.hasPath(".InnRoomCosts.VanillaInns"))
				int iInns = JDB.solveObj(".InnRoomCosts.VanillaInns")
				int iCount = JMap.count(iInns)
				string[] sInnArray = JMap.allKeysPArray(iInns)
				string sInn
				int i = 0
				while (i < iCount)
					sInn = sInnArray[i]
					VanillaInnsAdd(sInn)
					if (JDB.hasPath(".InnRoomCosts.VanillaInns." + sInn + ".Location"))
						SetInnLocation(sInn, JDB.solveStr(".InnRoomCosts.VanillaInns." + sInn + ".Location"))
					endif
					if (JDB.hasPath(".InnRoomCosts.VanillaInns." + sInn + ".InnRoomCost"))
						SetInnRoomCost(sInn, JDB.solveInt(".InnRoomCosts.VanillaInns." + sInn + ".InnRoomCost"))
					endif
					if (!bNewGame)
						if (JDB.hasPath(".InnRoomCosts.VanillaInns." + sInn + ".Visited"))
							SetVisited(sInn, JDB.solveInt(".InnRoomCosts.VanillaInns." + sInn + ".Visited"))
						endif
					endif
					i += 1
				endwhile
			endif
			if (JDB.hasPath(".InnRoomCosts.ModInns"))
				int iInns = JDB.solveObj(".InnRoomCosts.ModInns")
				int iCount = JMap.count(iInns)
				string[] sInnArray = JMap.allKeysPArray(iInns)
				string sInn
				int i = 0
				while (i < iCount)
					sInn = sInnArray[i]
					ModInnsAdd(sInn)
					if (JDB.hasPath(".InnRoomCosts.ModInns." + sInn + ".Location"))
						SetInnLocation(sInn, JDB.solveStr(".InnRoomCosts.ModInns." + sInn + ".Location"))
					endif
					if (JDB.hasPath(".InnRoomCosts.ModInns." + sInn + ".InnRoomCost"))
						SetInnRoomCost(sInn, JDB.solveInt(".InnRoomCosts.ModInns." + sInn + ".InnRoomCost"))
					endif
					if (!bNewGame)
						if (JDB.hasPath(".InnRoomCosts.ModInns." + sInn + ".Visited"))
							SetVisited(sInn, JDB.solveInt(".InnRoomCosts.ModInns." + sInn + ".Visited"))
						endif
					endif
					i += 1
				endwhile
			endif
			Save()
		endif
	endif

	;/
		If the VanillaInns list is empty, then there was no JSON file to load.
		Go ahead and populate the list from the FormList properties we got
		from the CK.
	/;
	if (VanillaInnsCount() == 0)
		Cell kCell
		Location kLoc
		string sInn
		int iDefaultCost = GetDefaultRoomCost()
		int iCount = InnCells.GetSize()
		int i = 0
		while (i < iCount)
			kCell = InnCells.GetAt(i) as Cell
			kLoc = InnLocations.GetAt(i) as Location
			sInn = kCell.GetName()
			if (!VanillaInnsHas(sInn))
				VanillaInnsAdd(sInn)
				SetInnLocation(sInn, kLoc.GetName())
			endif
			i += 1
		endwhile
		Save()
	endif
endfunction

;-------------------------------------------------------------------------------

bool function IsJContainersInstalled()
;	Debug.Trace("INN: IsJContainersInstalled()")
	int iVersion = -1
	if (SKSE.GetVersion() >= 2)
		iVersion = SKSE.GetPluginVersion("JContainers64")
		if (iVersion == -1)
			iVersion = SKSE.GetPluginVersion("JContainersVR")
		endif
	endif
;	Debug.Trace("INN:    SKSE.GetPluginVersion() returns " + iVersion)
	return iVersion != -1
endfunction

;-------------------------------------------------------------------------------

bool function IsPapyrusUtilInstalled()
;	Debug.Trace("INN: IsPapyrusUtilInstalled()")
;	Debug.Trace("INN:    SKSE.GetPluginVersion() returns " + SKSE.GetPluginVersion("papyrusutil"))
;	Debug.Trace("INN:    PapyrusUtil.GetVersion() returns " + PapyrusUtil.GetVersion())
	return PapyrusUtil.GetVersion() != 0
endfunction

;-------------------------------------------------------------------------------

event OnConfigClose()
	if (IsDirty())
		Save()
	endif
endevent

;-------------------------------------------------------------------------------

event OnConfigInit()
	ModName = "$InnRoomCosts"

	Pages = new string[3]
	Pages[0] = "$InnRoomCosts_ModOptions"
	Pages[1] = "$InnRoomCosts_Vanilla"
	Pages[2] = "$InnRoomCosts_Mods"
endevent

;-------------------------------------------------------------------------------

event OnGameReload()
	if (IsPapyrusUtilInstalled())
		LoadData()
		if (GetDebugMode())
			Debug.OpenUserLog(GetModName())
		endif
		parent.OnGameReload()
	else
		MsgNoPapyrusUtil.Show()

		; is this too harsh?
		Utility.Wait(0.1)	; wait for user to dismiss message box
		Debug.QuitGame()
	endif
endevent

;-------------------------------------------------------------------------------

event OnOptionHighlight(int aiOption)
	string text = ""
	if (CurrentPage == "$InnRoomCosts_ModOptions")
		if (aiOption == idDebugMode)
			text = "$InnRoomCosts_DebugMode_Desc"
		elseif (aiOption == idDefaultRoomCost)
			text = "$InnRoomCosts_DefaultRoomCost_Desc"
		elseif (aiOption == idUseCheckoutTime)
			text = "$InnRoomCosts_UseCheckoutTime_Desc"
		elseif (aiOption == idUseColoredLabels)
			text = "$InnRoomCosts_UseColoredLabels_Desc"
		elseif ((aiOption == idCheckoutTime) || (aiOption == idCheckoutTime2))
			text = "$InnRoomCosts_CheckoutTime_Desc"
		endif
	else
		string sInn = GetInnName(aiOption)
		text = "Inn: " + sInn + "\nLocation: " + GetInnLocation(sInn)
	endif
	SetInfoText(text)
endevent

;-------------------------------------------------------------------------------

event OnOptionSelect(int aiOption)
	if (CurrentPage == "$InnRoomCosts_ModOptions")
		if (aiOption == idDebugMode)
			bool bValue = !GetDebugMode()
			SetToggleOptionValue(idDebugMode, bValue)
			SetDebugMode(bValue)
			if (bValue)
				Debug.OpenUserLog(GetModName())
			else
				Debug.CloseUserLog(GetModName())
			endif
		elseif (aiOption == idUseCheckoutTime)
			bool bValue = !UseCheckoutTime()
			SetToggleOptionValue(idUseCheckoutTime, bValue)
			SetUseCheckoutTime(bValue)
	
			int iFlags = OPTION_FLAG_DISABLED
			if (bValue)
				iFlags = OPTION_FLAG_NONE
			endif
			SetOptionFlags(idCheckoutTime, iFlags)
		elseif (aiOption == idUseColoredLabels)
			bool bValue = !UseColoredLabels()
			SetToggleOptionValue(idUseColoredLabels, bValue)
			SetUseColoredLabels(bValue)
		endif
	endif
endevent

;-------------------------------------------------------------------------------

event OnOptionSliderAccept(int aiOption, float afValue)
	if (CurrentPage == "$InnRoomCosts_ModOptions")
		if (aiOption == idCheckoutTime)
			SetCheckoutTime(afValue)
			SetSliderOptionValue(idCheckoutTime, afValue, "{2}")
			SetTextOptionValue(idCheckoutTime2, FormatTime(afValue))
		elseif (aiOption == idDefaultRoomCost)
			SetDefaultRoomCost(afValue as int)
			SetSliderOptionValue(idDefaultRoomCost, afValue, "{0}")
		endif
	else
		if (aiOption == idDefaultRoomCost)
			SetDefaultRoomCost(afValue as int)
		else
			SetInnRoomCost(GetInnName(aiOption), afValue as int)
		endif
		SetSliderOptionValue(aiOption, afValue, "{0}")
	endif
endevent

;-------------------------------------------------------------------------------

event OnOptionSliderOpen(int aiOption)
	if (CurrentPage == "$InnRoomCosts_ModOptions")
		if (aiOption == idCheckoutTime)
			SetSliderDialogStartValue(GetCheckoutTime())
			SetSliderDialogDefaultValue(GetDefaultCheckoutTime())
			SetSliderDialogRange(0.0, 23.75)
			SetSliderDialoginterval(0.25)
		elseif (aiOption == idDefaultRoomCost)
			SetSliderDialogStartValue(GetDefaultRoomCost())
			SetSliderDialogDefaultValue(GetDefaultRoomCostDefault())
			SetSliderDialogRange(10.0, 200.0)
			SetSliderDialoginterval(5.0)
		endif
	else
		float fValue
		float fDefaultValue
		if (aiOption == idDefaultRoomCost)
			fValue = GetDefaultRoomCost()
			fDefaultValue = GetDefaultRoomCostDefault()
		else
			fValue = GetInnRoomCost(GetInnName(aiOption))
			fDefaultValue = GetDefaultRoomCost()
		endif
		SetSliderDialogStartValue(fValue)
		SetSliderDialogDefaultValue(fDefaultValue)
		SetSliderDialogRange(10.0, 500.0)
		SetSliderDialoginterval(5.0)
	endif
endevent

;-------------------------------------------------------------------------------

event OnPageReset(string asPage)
	if (asPage == "$InnRoomCosts_ModOptions")
		SetCursorFillMode(TOP_TO_BOTTOM)
		idDebugMode = AddToggleOption("$InnRoomCosts_DebugMode", GetDebugMode())
		idUseColoredLabels = AddToggleOption("$InnRoomCosts_UseColoredLabels", UseColoredLabels())
		AddEmptyOption()
		idDefaultRoomCost = AddSliderOption("$InnRoomCosts_DefaultRoomCost", GetDefaultRoomCost() as float, "{0}")
		AddEmptyOption()
		idUseCheckoutTime = AddToggleOption("$InnRoomCosts_UseCheckoutTime", UseCheckoutTime())
		int iFlags = OPTION_FLAG_DISABLED
		if (UseCheckoutTime())
			iFlags = OPTION_FLAG_NONE
		endif
		idCheckoutTime = AddSliderOption("$InnRoomCosts_CheckoutTime", GetCheckoutTime(), "{2}", iFlags)
		idCheckoutTime2 = AddTextOption("", FormatTime(GetCheckoutTime()), OPTION_FLAG_DISABLED)
	elseif (asPage == "$InnRoomCosts_Vanilla")
		SetTitleText(asPage)
		SetCursorFillMode(LEFT_TO_RIGHT)
		int iCount = VanillaInnsCount()
		idInn = Utility.CreateIntArray(iCount)
		string sInn
		int i = 0
		while (i < iCount)
			sInn = VanillaInnsGet(i)
			idInn[i] = AddSliderOption(GetInnDisplayName(sInn), GetInnRoomCost(sInn) as float, "{0}")
			i += 1
		endwhile
	elseif (asPage == "$InnRoomCosts_Mods")
		SetTitleText(asPage)
		SetCursorFillMode(LEFT_TO_RIGHT)
		int iCount = ModInnsCount()
		idInn = Utility.CreateIntArray(iCount)
		string sInn
		int i = 0
		while (i < iCount)
			sInn = ModInnsGet(i)
			idInn[i] = AddSliderOption(GetInnDisplayName(sInn), GetInnRoomCost(sInn) as float, "{0}")
			i += 1
		endwhile
	endif
endevent

;-------------------------------------------------------------------------------

event OnVersionUpdate(int a_version)
	; a_version is the new version, CurrentVersion is the old version
	if (CurrentVersion == 0)
		; The user has started a new game.
	elseif (CurrentVersion < 2) 
		InnScript.Migration(CurrentVersion)
		OnConfigInit()
	endif
endevent
