scriptname INN_RoomCostsScript extends Quest

import INN_Data

FormList property RentRoomFactionList auto
{ A list of factions for innkeepers that rent rooms. }

FormList property AffectedQuests auto
{ A list of quests that cache the room cost global variable. }

GlobalVariable property RoomCost auto
{ The room cost variable used by the vanilla DialogueGeneric quest. }

INN_RoomCostsConfig Config = none
bool bMonitorDialogue = false
Location kLocation = none
MODExtendedStayRefresher kExtendedStay = none
int iIndent = 0

;-------------------------------------------------------------------------------

Actor function GetPlayerDialogueTarget()
	Actor kPlayerRef = Game.GetPlayer()
	Actor kTargetRef = none
	Actor kNthRef    = none

	Cell kCell       = kPlayerRef.GetParentCell()
	int iType        = 43 ; kNPC = 43
	int iIndex       = kCell.GetNumRefs(iType) 
	
	while ((iIndex > 0) && (kTargetRef == none))
		iIndex -= 1
		kNthRef = kCell.GetNthRef( iIndex, iType ) as Actor
		if ((kNthRef != kPlayerRef) && kNthRef.IsInDialogueWithPlayer())
			kTargetRef = kNthRef
		endIf
	endwhile
	
	return kTargetRef
endfunction

;-------------------------------------------------------------------------------

function Init(bool abNewGame)
	if (Config == none)
		Config = Quest.GetQuest("INN_RoomCostsConfig") as INN_RoomCostsConfig
	endif

	Log("Init(" + abNewGame + ")")
;/
	int iCount = Config.VanillaInnsCount()
	if (iCount == 0)
		Cell kCell
		GlobalVariable kRoomCost
		Location kLoc
		string sInn
		int i = 0
		while (i < InnCells.GetSize())
			kCell = InnCells.GetAt(i) as Cell
			kRoomCost = InnRoomCosts.GetAt(i) as GlobalVariable
			kLoc = InnLocations.GetAt(i) as Location
			sInn = kCell.GetName()
			if (!Config.VanillaInnsHas(sInn))
				Config.VanillaInnsAdd(sInn)
				Config.SetInnRoomCost(sInn, kRoomCost.GetValueInt())
				Config.SetInnLocation(sInn, kLoc.GetName())
			endif
			i += 1
		endwhile
		Config.JCSaveData()
	endif
/;
	if (Game.GetModByName("ExtendedStay.esp") != 255)
		if (kExtendedStay == none)
			kExtendedStay = Game.GetFormFromFile(0x000012cb, "ExtendedStay.esp") as MODExtendedStayRefresher
		endif
	else
		kExtendedStay = none
	endif

	RegisterForSingleUpdate(2.0)
endfunction

;-------------------------------------------------------------------------------

bool function IsInnKeeper(Actor akActor)
	bool bIsInnKeeper = false
	Faction kFaction
	int iCount = RentRoomFactionList.GetSize()
	int i = 0
	while ((i < iCount) && !bIsInnKeeper)
		kFaction = RentRoomFactionList.GetAt(i) as Faction
		bIsInnKeeper = akActor.IsInFaction(kFaction)
		i += 1
	endwhile
	return bIsInnKeeper
endfunction

;-------------------------------------------------------------------------------

bool function IsVR()
	return Game.GetModByName("SkyrimVR.esm") != 255
endfunction

;-------------------------------------------------------------------------------

function Log(string asText, bool abNotify = false)
	if (GetDebugMode())
		if (abNotify)
			Debug.Notification(asText)
		endif
		string sMsg = asText
		int i = 0
		while (i < iIndent)
			sMsg = "   " + sMsg
			i += 1
		endwhile
		Debug.TraceUser(GetModName(), sMsg, 0)
		if (IsVR())
			Utility.Wait(0.01)
		endif
	endif
endfunction

;-------------------------------------------------------------------------------

function LogAffectedQuests()
	Log("LogAffectedQuests()")
	iIndent += 1

	Quest kQuest
	int i = 0
	while (i < AffectedQuests.GetSize())
		kQuest = AffectedQuests.GetAt(i) as Quest
		Log(kQuest.GetID())
		i += 1
	endwhile

	iIndent -= 1
endfunction

;-------------------------------------------------------------------------------

function LogLocation(Location akLoc, string asName)
	string text = "unknown"
	if (akLoc != none)
		text = akLoc.GetName()
	endif
	Log(asName + ".GetName() -> " + text)
endfunction

;-------------------------------------------------------------------------------

function LocationChanged(Location akOldLoc, Location akNewLoc)
	Log("LocationChanged()")
	iIndent += 1

	LogLocation(akOldLoc, "akOldLoc")
	LogLocation(akNewLoc, "akNewLoc")

	if (bMonitorDialogue)
		UnregisterForMenu("Dialogue Menu")
		bMonitorDialogue = false
		kLocation = none
	endif

	Actor kPlayer = Game.GetPlayer()
	Cell kCurrentCell = kPlayer.GetParentCell()
	if (kCurrentCell.IsInterior())
		string sCellName = kCurrentCell.GetName()
		bool bIsKnownInn = VanillaInnsHas(sCellName) || ModInnsHas(sCellName)

		if (bIsKnownInn)
			Log("You already know this inn; setting the room cost.")
			int iRoomCost = GetInnRoomCost(sCellName)
			Log("setting room cost to " + iRoomCost)
			RoomCost.SetValueInt(iRoomCost)
		else
			Log("You don't know of any inn at this location.")
			int iDefaultRoomCost = GetDefaultRoomCost()
			Log("Setting room cost to " + iDefaultRoomCost)
			RoomCost.SetValue(iDefaultRoomCost)
			kLocation = akOldLoc
		endif

		; monitor dialogue with NPCs to see if this is a new inn
		RegisterForMenu("Dialogue Menu")
		bMonitorDialogue = true

		LogAffectedQuests()
		Quest kQuest
		int iCount = AffectedQuests.GetSize()
		int i = 0
		while (i < iCount)
			kQuest = AffectedQuests.GetAt(i) as Quest
			kQuest.UpdateCurrentInstanceGlobal(RoomCost)
			i += 1
		endwhile

		if (kExtendedStay != none)
			float fRoomCost = RoomCost.GetValue()
			kExtendedStay.RoomCost3Day.SetValue(3.0 * fRoomCost)
			kExtendedStay.RoomCost7Day.SetValue(7.0 * fRoomCost)
			kExtendedStay.RoomCost30Day.SetValue(30.0 * fRoomCost)
			kExtendedStay.Refresh()
		endif
	endif

	iIndent -= 1
endfunction

;-------------------------------------------------------------------------------

function Migration(int aiOldVersion)
	if (aiOldVersion < 2)
		RentRoomFactionList = Game.GetFormFromFile(0x0005f360, "InnRoomCosts.esp") as FormList
	endif
endfunction

;-------------------------------------------------------------------------------

event OnMenuOpen(String MenuName)
	Log("OnMenuOpen(" + MenuName + ")")
	iIndent += 1

	Actor kRef = GetPlayerDialogueTarget()
	if (kRef != none)
		Log("kRef = " + kRef.GetName())
		if (IsInnKeeper(kRef))
			Log("kRef is an innkeeper")
			Cell kCell = Game.GetPlayer().GetParentCell()
			string sInn = kCell.GetName()
			bool bIsKnownInn = VanillaInnsHas(sInn) || ModInnsHas(sInn)
			Log("sInn = '" + sInn + "'")
			Log("bIsKnownInn = " + bIsKnownInn)
			if (!bIsKnownInn)
				Log("New inn discovered: " + sInn, true)
				ModInnsAdd(sInn)
				if (kLocation != none)
					SetInnLocation(sInn, kLocation.GetName())
;				else
;					SetInnLocation(sInn, "unknown")
				endif
			endif
			SetVisited(sInn, true)
			UnregisterForMenu("Dialogue Menu")
			bMonitorDialogue = false
			kLocation = none
		else
			Log("kRef is not an innkeeper")
		endif
	else
		Log("kRef is none")
	endif

	iIndent -= 1
endevent

;-------------------------------------------------------------------------------

event OnUpdate()
	Patch()
endevent

;-------------------------------------------------------------------------------

function Patch()
	Log("Patch()")
	iIndent += 1

	Log("before revert")
	LogAffectedQuests()

	;/
		Revert the FormList(s) to remove anything that got added via previous
		calls to this function.  This is necessary to handle the case where
		a mod was uninstalled/deactivated in the middle of a play through.
		If that didn't happen, then the quests will be added back in below.
	/;
	AffectedQuests.Revert()
	RentRoomFactionList.Revert()

	Log("after revert")
	LogAffectedQuests()

	;/
		Patch for ETaC version 0.4.4
	/;
	if (Game.GetModByName("ETaC - RESOURCES.esm") != 255)
		Log("ETaC detected")
		iIndent += 1

		Actor kVarin = none
		if (Game.GetModByName("ETaC - Complete.esp") != 255)
			Log("using 'ETaC - Complete.esp'")
			kVarin = Game.GetFormFromFile(0x00990005, "ETaC - Complete.esp") as Actor
		elseif (Game.GetModByName("ETaC - Darkwater Crossing.esp") != 255)
			Log("using 'ETaC - Darkwater Crossing.esp'")
			kVarin = Game.GetFormFromFile(0x00990005, "ETaC - Darkwater Crossing.esp") as Actor
		endif
		if (kVarin == none)
			Log("Varin not found; not using Darkwater Crossing")
		else
			Log("adding Varin to faction JobRentRoomFaction")
			Faction kJobRentRoomFaction = RentRoomFactionList.GetAt(0) as Faction
			;/
				No need to check first; this function will do nothing if
				he is already in the faction.
			/;
			kVarin.AddToFaction(kJobRentRoomFaction)

			;/
				Add the dialog quest to the list so that it can be updated
				whenever the RoomCost global variable is changed.
			/;
			Log("adding MJBDarkwaterCrossingDialogue to list of affected quests")
			Form kQuest = Quest.GetQuest("MJBDarkwaterCrossingDialogue") as Form
			AffectedQuests.AddForm(kQuest)
		endif

		iIndent -= 1
	endif

	;/
		Patch for Beyond Skyrim - Bruma version 1.3.3

		NOTE:	This mod requires additional patching of dialogue that cannot be done in
				script.  This is why the plugin "InnRoomCosts - BS Bruma Patch.esp" is
				required to complete this patch. 
	/;
	if (Game.GetModByName("BSHeartland.esm") != 255)
		Log("Beyond Skyrim - Bruma detected")
		iIndent += 1

		;/
			Add CYRJobRentRoomFaction to RentRoomFactionList so that the inns in
			this mod may be properly discovered at run time.
		/;
		Log("Adding CYRJobRentRoomFaction to RentRoomFactionList")
		Form kCYRJobRentRoomFaction = Game.GetFormFromFile(0x0005bbdc, "BSHeartland.esm")
		RentRoomFactionList.AddForm(kCYRJobRentRoomFaction)

		;/
			Add the dialog quest to the list so that it can be updated
			whenever the RoomCost global variable is changed.
		/;
		Log("adding CYRGenericDialogue to list of affected quests")
		Form kQuest = Quest.GetQuest("CYRGenericDialogue") as Form
		AffectedQuests.AddForm(kQuest)

		iIndent -= 1
	endif

	Log("after patching")
	LogAffectedQuests()

	iIndent -= 1
endfunction
