Scriptname RentRoomScript extends Actor  Conditional
{script for anyone who rents a room}

ObjectReference Property Bed  Auto  
{bed to rent}

WIFunctionsScript Property WI Auto
{Pointer to WIFunctionsScript attached to WI quest}

;-------------------------------------------------------------------------------

function ClearRoom()
	if (Game.GetModByName("InnRoomCosts.esp") != 255)
		if (INN_Data.GetDebugMode())
			INN_RoomCostsScript INNScript = Game.GetFormFromFile(0x00000800, "InnRoomCosts.esp") as INN_RoomCostsScript
			if (INNScript != none)
				float fDays = Utility.GetCurrentGameTime()
				float fTimeOfDay = (fDays - Math.Floor(fDays)) * 24.0
				InnScript.Log("Room rental expired (time = " + fTimeOfDay + ")")
			endif
		endif
	endif

	; clear ownership - either rental expired or I died
	Bed.SetActorOwner((self as Actor).GetActorBase())
	UnregisterForUpdateGameTime()
	; used to conditionalize innkeeper dialogue
	SetActorValue("Variable09", 0.0)
endfunction

;-------------------------------------------------------------------------------

function FreeRoom(Float afRentHours)
	InternalRentRoom(1, none, true)
endfunction

;-------------------------------------------------------------------------------

float function GetCheckoutTime(int aiNumDays)
	float fCheckoutTime = 24.0 * aiNumDays

	if (Game.GetModByName("InnRoomCosts.esp") != 255)
		if (INN_Data.UseCheckoutTime())
			float fDays = Utility.GetCurrentGameTime()
			float fHourOfDay = (fDays - Math.Floor(fDays)) * 24.0
			fCheckoutTime += INN_Data.GetCheckoutTime() - fHourOfDay
	
			if (INN_Data.GetDebugMode())
				INN_RoomCostsScript INNScript = Game.GetFormFromFile(0x00000800, "InnRoomCosts.esp") as INN_RoomCostsScript
				if (INNScript != none)
					INNScript.Log("GetCheckoutTime()")
					INNScript.Log("   aiNumDays         = " + aiNumDays)
					INNScript.Log("   fDays             = " + fDays)
					INNScript.Log("   fHourOfDay        = " + fHourOfDay)
					INNScript.Log("   checkout time     = " + INN_Data.GetCheckoutTime())
					INNScript.Log("   hours bed owned   = " + fCheckoutTime)
				endif
			endif
		endif
	endif

	return fCheckoutTime
endfunction

;-------------------------------------------------------------------------------

function InternalRentRoom(int aiNumDays, Form akGold, bool abFreeRoom = false)
	Bed.SetActorOwner(Game.GetPlayer().GetActorBase())

	GlobalVariable kRoomCost = Game.GetFormFromFile(0x0009cc98, "Skyrim.esm") as GlobalVariable
	int iCost = kRoomCost.GetValueInt() * aiNumDays

	if (Game.GetModByName("InnRoomCosts.esp") != 255)
		if (INN_Data.GetDebugMode())
			INN_RoomCostsScript INNScript = Game.GetFormFromFile(0x00000800, "InnRoomCosts.esp") as INN_RoomCostsScript
			if (INNScript != none)
				INNScript.Log("InternalRentRoom()")
				INNScript.Log("   aiNumDays    = " + aiNumDays)
				INNScript.Log("   cost per day = " + kRoomCost.GetValueInt())
				INNScript.Log("   iCost        = " + iCost)
			endif
		endif
	endif

	RegisterForSingleUpdateGameTime(GetCheckoutTime(aiNumDays))
	if (!abFreeRoom)
		Game.GetPlayer().RemoveItem(akGold, iCost)
	endif

	; used to conditionalize innkeeper dialogue
	SetActorValue("Variable09", 1.0)
	ShowPlayerRoom()
endfunction

;-------------------------------------------------------------------------------

event OnUpdateGameTime()
	ClearRoom()
endevent

;-------------------------------------------------------------------------------

event OnDeath(Actor akKiller)
	ClearRoom()
endevent

;-------------------------------------------------------------------------------

function RentRoom(DialogueGenericScript pQuestScript)
	InternalRentRoom(1, pQuestScript.Gold)
endfunction

;-------------------------------------------------------------------------------

function RentRoom3Day(ModExtendedStayVariables pQuestScript)
	InternalRentRoom(3, pQuestScript.Gold)
endfunction

;-------------------------------------------------------------------------------

function RentRoom7Day(ModExtendedStayVariables pQuestScript)
	InternalRentRoom(7, pQuestScript.Gold)
endfunction

;-------------------------------------------------------------------------------

function RentRoom30Day(ModExtendedStayVariables pQuestScript)
	InternalRentRoom(30, pQuestScript.Gold)
endfunction

;-------------------------------------------------------------------------------

function ShowPlayerRoom()
	bool bShow = true

	if (Game.GetModByName("TavernAIFix.esp") != 255)
		GlobalVariable ShowRoomNewerShow = Game.GetFormFromFile(0x00000807, "TavernAIFix.esp") as GlobalVariable
		if (ShowRoomNewerShow && ShowRoomNewerShow.GetValue() == 1.0)
			bShow = false
		endif

		if (bShow)
			FormList ShowRoomExcludeList = Game.GetFormFromFile(0x00000806, "TavernAIFix.esp") as FormList
			if (ShowRoomExcludeList)
				if (ShowRoomExcludeList.HasForm(GetBaseObject()))
					bShow = false
				endif
				ShowRoomExcludeList.AddForm(GetBaseObject())
			endif
		endif
	endif

	if (bShow)
		WI.ShowPlayerRoom(self, Bed)
	endif
endfunction
